import axios from 'axios';
import Cookies from 'js-cookie';

// create an axios instance
const service = axios.create({
  baseURL: '/', // url = base url + request url
  withCredentials: true, // send cookies when cross-domain requests
  timeout: 1800000, // request timeout
});

// request interceptor
service.interceptors.request.use(
  (config) => {
    // do something before request is sent

    const token = Cookies.get('id');
    if (token) {
      config.headers = {
        AuthId: token,
        Authorization: `Bearer ${token}`,
      };
    }
    config.responseType = 'blob';
    return config;
  },
  (error) => {
    // do something with request error
    console.log(error); // for debug
    return Promise.reject(error);
  },
);

export default service;
