import Vue from 'vue';
import './assets/css/reset.css';
import Cookies from 'js-cookie';
import ElementUI from 'element-ui';
import BaiduMap from 'vue-baidu-map';
import 'element-ui/lib/theme-chalk/index.css';
import _ from 'lodash';
import moment from 'moment';


import VueVideoPlayer from 'vue-video-player';
import axios from 'axios';
import store from '@/store';
import App from './App.vue';
import router from './router';

// require videojs style
import 'video.js/dist/video-js.css';
import './utils/dialog';
import 'vue-video-player/src/custom-theme.css';

Vue.directive('real-img', async (el, binding) => { // 指令名称为：real-img
  const imgURL = binding.value;// 获取图片地址
  if (imgURL) {
    const exist = await imageIsExist(imgURL);
    if (exist) {
      el.setAttribute('src', imgURL);
    }
  }
});

/**
 * 检测图片是否存在
 * @param url
 */
let imageIsExist = function (url) {
  return new Promise((resolve) => {
    let img = new Image();
    img.onload = function () {
      if (this.complete == true) {
        resolve(true);
        img = null;
      }
    };
    img.onerror = function () {
      resolve(false);
      img = null;
    };
    img.src = url;
  });
};

Vue.use(VueVideoPlayer);
const hls = require('videojs-contrib-hls');

Vue.use(hls);

Vue.use(ElementUI);
Vue.use(BaiduMap, {
  ak: 'AZzMTSKLP4lVHphRauTBdINZUFnpWTcu',
});
Object.defineProperty(Vue.prototype, '$moment', { value: moment });
Object.defineProperty(Vue.prototype, '$_', { value: _ });


var _hmt = _hmt || [];
window._hmt = _hmt; // 必须把_hmt挂载到window下，否则找不到
(function () {
  const hm = document.createElement('script');
  hm.src = 'https://hm.baidu.com/hm.js?264c2115edc746cbdd76f5fa44b60c26';
  const s = document.getElementsByTagName('script')[0];
  s.parentNode.insertBefore(hm, s);
}());


Vue.config.productionTip = false;
router.beforeEach((to, from, next) => {
  window.scrollTo(0, 0);
  if (to.matched.some(record => record.meta.requireAuth)) {
    // 判断该路由是否需要登录权限
    if (Cookies.get('id')) { // 判断当前的token是否存在
      if (_hmt) {
        if (to.path) {
          _hmt.push(['_trackPageview', `/#${to.fullPath}`]);
        }
      }
      next();
    } else {
      // next();
      next({
        path: '/login',
        query: { redirect: to.fullPath },
        // 将跳转的路由path作为参数，登录成功后跳转到该路由
      });
    }
  } else {
    next();
  }
});
axios.interceptors.request.use(
  (config) => {
    config.baseURL = '/';
    config.withCredentials = true; // 允许携带token ,这个是解决跨域产生的相关问题
    config.timeout = 60000;
    const token = Cookies.get('id');
    const companyId = Cookies.get('companyId');
    if (token) {
      config.headers = {
        companyId,
        AuthId: token,
        Authorization: `Bearer ${token}`,
      };
    }
    if (config.url === 'refresh') {
      config.headers = {
        Authorization: `Bearer ${token}`,
      };
    }
    return config;
  },
  error => Promise.reject(error),
);


router.afterEach((to, from, next) => {
  window.scrollTo(0, 0);
});

const permission = {
  inserted(el, binding, vnode) {
    const { value } = binding;
    const roles = JSON.parse(localStorage.getItem('permission'));
    if (value && value instanceof Array && value.length > 0) {
      const permissionRoles = value;

      const hasPermission = roles ? roles.some(role => permissionRoles.includes(role)) : false;
      if (!hasPermission) {
        el.parentNode && el.parentNode.removeChild(el);
      }
    } else {
      throw new Error('need roles! Like v-permission="[\'admin\',\'editor\']"');
    }
  },
};
Vue.directive('permission', permission);
new Vue({
  router,
  store,
  render: h => h(App),
}).$mount('#app');
