import axios from 'axios';
import { MessageBox, Message } from 'element-ui';
import store from '@/store';
import router from '@/router';
import { getToken } from './cookie';


// request interceptor
axios.interceptors.request.use(
  (config) => {
    // do something before request is sent

    if (store.getters.token) {
      // eslint-disable-next-line no-param-reassign
      config.headers.Authorization = '22222';
    }
    return config;
  },
  (error) => {
    // do something with request error
    console.log(error); // for debug
    return Promise.reject(error);
  },
);

// response interceptor
axios.interceptors.response.use(
  /**
   * If you want to get http information such as headers or status
   * Please return  response => response
  */

  /**
   * Determine the request status by custom code
   * Here is just an example
   * You can also judge the status by HTTP Status Code
   */
  (response) => {
    const res = response.data;
    // -1 通用错误提示
    if (res.code === -1) {
      Message({
        message: res.message || 'Error',
        type: 'error',
        duration: 3 * 1000,
      });
      return Promise.reject(new Error(res.message || 'Error'));
    }
    return res;
  },
  (error) => {
    if (error.response.status === 401) {
      MessageBox.confirm('用户信息过期，请重新登录', '提示', {
        confirmButtonText: '重登录',
        cancelButtonText: '取消',
        type: 'warning',
      }).then(() => {
        store.dispatch('user/resetToken').then(() => {
          location.reload();
        });
      });
    } else if (error.response.status === 403) {
      router.push({ path: '/403' });
    } else {
      Message({
        message: error.message,
        type: 'error',
        duration: 3 * 1000,
      });
    }
    return Promise.reject(error);
  },
);
